@@ -58,7 +58,6 @@ public:
     } else {
       data[index.pos] = x;
     }
-    summary_map.insert(symbol);
   }
 
   virtual int size() const {
@@ -69,6 +68,11 @@ public:
     return nr;
   }
 
+  void input_summarised(const SymbolString& symbol, SummarisedVariable x) {
+    input(symbol, x);
+    summary_map.insert(symbol);
+  }
+
 public:
   void clear() {}
 
