@@ -12,7 +12,7 @@ public:
   virtual SEXP process(const GroupedDataFrame& gdf) {
     int ng = gdf.ngroups();
 
-    Vector<RTYPE> out = no_init(gdf.nrows());
+    Vector<RTYPE> out(no_init(gdf.nrows()));
     GroupedDataFrame::group_iterator git = gdf.group_begin();
     for (int i = 0; i < ng; i++, ++git) {
       static_cast<Derived&>(*this).process_slice(out, *git, *git);
@@ -23,7 +23,7 @@ public:
   virtual SEXP process(const RowwiseDataFrame& gdf) {
     int ng = gdf.ngroups();
 
-    Vector<RTYPE> out = no_init(gdf.nrows());
+    Vector<RTYPE> out(no_init(gdf.nrows()));
     RowwiseDataFrame::group_iterator git = gdf.group_begin();
     for (int i = 0; i < ng; i++, ++git) {
       static_cast<Derived&>(*this).process_slice(out, *git, *git);
@@ -33,7 +33,7 @@ public:
 
   virtual SEXP process(const SlicingIndex& index) {
     int nrows = index.size();
-    Vector<RTYPE> out = no_init(nrows);
+    Vector<RTYPE> out(no_init(nrows));
     NaturalSlicingIndex fake(nrows);
     static_cast<Derived&>(*this).process_slice(out, index, fake);
     return out;
