@@ -38,7 +38,8 @@ test_that("cbind uses shallow copies", {
     int = 1:10,
     num = rnorm(10),
     cha = letters[1:10],
-    stringsAsFactors = FALSE)
+    stringsAsFactors = FALSE
+  )
   df2 <- data.frame(
     log = sample(c(T, F), 10, replace = TRUE),
     dat = seq.Date(Sys.Date(), length.out = 10, by = "day"),
@@ -115,11 +116,17 @@ test_that("bind_rows ignores NULL", {
   expect_equal(bind_rows(list(df, NULL)), df)
 })
 
-test_that("bind_rows only accepts data frames or vectors", {
+test_that("bind_rows only accepts data frames or named vectors", {
   ll <- list(1:5, rlang::get_env())
   expect_error(
     bind_rows(ll),
-    "Argument 2 is a list, must contain atomic vectors",
+    "Argument 1 must have names",
+    fixed = TRUE
+  )
+  ll <- list(tibble(a = 1:5), rlang::get_env())
+  expect_error(
+    bind_rows(ll),
+    "Argument 2 must be a data frame or a named atomic vector, not a environment",
     fixed = TRUE
   )
 })
@@ -256,17 +263,23 @@ test_that("bind_rows doesn't promote integer/numeric to factor", {
 
 
 test_that("bind_rows preserves timezones #298", {
-  dates1 <- data.frame(ID = c("a", "b", "c"),
+  dates1 <- data.frame(
+    ID = c("a", "b", "c"),
     dates = structure(c(-247320000, -246196800, -245073600),
       tzone = "GMT",
-      class = c("POSIXct", "POSIXt")),
-    stringsAsFactors = FALSE)
+      class = c("POSIXct", "POSIXt")
+    ),
+    stringsAsFactors = FALSE
+  )
 
-  dates2 <- data.frame(ID = c("d", "e", "f"),
+  dates2 <- data.frame(
+    ID = c("d", "e", "f"),
     dates = structure(c(-243864000, -242654400, -241444800),
       tzone = "GMT",
-      class = c("POSIXct", "POSIXt")),
-    stringsAsFactors = FALSE)
+      class = c("POSIXct", "POSIXt")
+    ),
+    stringsAsFactors = FALSE
+  )
 
   alldates <- bind_rows(dates1, dates2)
   expect_equal(attr(alldates$dates, "tzone"), "GMT")
@@ -288,7 +301,6 @@ test_that("bind_rows handles all NA columns (#493)", {
   res <- bind_rows(mydata)
   expect_true(is.na(res$x[1]))
   expect_is(res$x, "factor")
-
 })
 
 test_that("bind_rows handles complex. #933", {
@@ -376,7 +388,6 @@ test_that("bind handles POSIXct of different tz ", {
 
   res <- bind_rows(df1, df2, df3)
   expect_equal(attr(res$date, "tzone"), "UTC")
-
 })
 
 test_that("bind_rows() creates a column of identifiers (#1337)", {
@@ -549,12 +560,14 @@ test_that("bind_rows() fails with unnamed vectors", {
 })
 
 test_that("bind_rows() handles rowwise vectors", {
-  expect_warning(regex = "character and factor",
+  expect_warning(
+    regex = "character and factor",
     tbl <- bind_rows(
       tibble(a = "foo", b = "bar"),
       c(a = "A", b = "B"),
       set_names(factor(c("B", "B")), c("a", "b"))
-    ))
+    )
+  )
   expect_identical(tbl, tibble(a = c("foo", "A", "B"), b = c("bar", "B", "B")))
 
   id_tbl <- bind_rows(a = c(a = 1, b = 2), b = c(a = 3, b = 4), .id = "id")
@@ -565,23 +578,30 @@ test_that("bind_rows() accepts lists of dataframe-like lists as first argument",
   expect_identical(bind_rows(list(list(a = 1, b = 2))), tibble(a = 1, b = 2))
 })
 
+test_that("columns that are OBJECT but have NULL class are handled gracefully (#3349)", {
+  mod <- lm(y ~ ., data = freeny)
+  data <- model.frame(mod)
+  data_list <- list(data, data)
+  res <- bind_rows(data_list)
+  expect_equal(names(res), names(data))
+})
 
 # Vectors ------------------------------------------------------------
 
 test_that("accepts named columns", {
   expect_identical(bind_cols(a = 1:2, b = 3:4), tibble(a = 1:2, b = 3:4))
-  expect_equal(bind_cols(!!! mtcars), as_tibble(mtcars))
+  expect_equal(bind_cols(!!!mtcars), as_tibble(mtcars))
 })
 
 test_that("uncompatible sizes fail", {
   expect_error(
     bind_cols(a = 1, mtcars),
-    "Argument 2 must be length 32, not 1",
+    "Argument 2 must be length 1, not 32",
     fixed = TRUE
   )
   expect_error(
-    bind_cols(mtcars, a = 1),
-    "Argument 2 must be length 1, not 32",
+    bind_cols(mtcars, a = 1:3),
+    "Argument 2 must be length 32, not 3",
     fixed = TRUE
   )
 })
@@ -593,7 +613,7 @@ test_that("unnamed vectors fail", {
     fixed = TRUE
   )
   expect_error(
-    bind_cols(!!! list(1:2)),
+    bind_cols(!!!list(1:2)),
     "Argument 1 must have names",
     fixed = TRUE
   )
@@ -603,3 +623,9 @@ test_that("supports NULL values", {
   expect_identical(bind_cols(a = 1, NULL, b = 2, NULL), tibble(a = 1, b = 2))
 })
 
+test_that("bind_cols handles unnamed list (#3402)", {
+  expect_identical(
+    bind_cols(list(1, 2)),
+    bind_cols(list(V1 = 1, V2 = 2))
+  )
+})
