@@ -30,9 +30,10 @@ test_that("cases must yield compatible lengths", {
   expect_error(
     case_when(
       c(TRUE, FALSE) ~ 1,
-      c(FALSE, TRUE, FALSE) ~ 2
+      c(FALSE, TRUE, FALSE) ~ 2,
+      c(FALSE, TRUE, FALSE, NA) ~ 3
     ),
-    "LHS of case 1 (`c(TRUE, FALSE)`) must be length 3 (the longest input) or one, not 2",
+    "`c(FALSE, TRUE, FALSE) ~ 2`, `c(FALSE, TRUE, FALSE, NA) ~ 3` must be length 2 or one, not 3, 4",
     fixed = TRUE
   )
 
@@ -41,7 +42,7 @@ test_that("cases must yield compatible lengths", {
       c(TRUE, FALSE) ~ 1:3,
       c(FALSE, TRUE) ~ 1:2
     ),
-    "RHS of case 1 (1:3) must be length 2 (the first output) or one, not 3",
+    "`c(TRUE, FALSE) ~ 1:3` must be length 2 or one, not 3",
     fixed = TRUE
   )
 })
@@ -80,3 +81,47 @@ test_that("missing values can be replaced (#1999)", {
     c(1, 2, NA, 0)
   )
 })
+
+test_that("NA conditions (#2927)", {
+  expect_equal(
+    case_when(
+      c(TRUE, FALSE, NA) ~ 1:3,
+      TRUE ~ 4L
+    ),
+    c(1L, 4L, 4L)
+  )
+})
+
+test_that("atomic conditions (#2909)", {
+  expect_equal(
+    case_when(
+      TRUE ~ 1:3,
+      FALSE ~ 4:6
+    ),
+    1:3
+  )
+  expect_equal(
+    case_when(
+      NA ~ 1:3,
+      TRUE ~ 4:6
+    ),
+    4:6
+  )
+})
+
+test_that("zero-length conditions and values (#3041)", {
+  expect_equal(
+    case_when(
+      TRUE ~ integer(),
+      FALSE ~ integer()
+    ),
+    integer()
+  )
+  expect_equal(
+    case_when(
+      logical() ~ 1,
+      logical() ~ 2
+    ),
+    numeric()
+  )
+})
